@@ -77,7 +77,7 @@ get_versiondep_info_x86(void)
 	 * linux-2.6.26.
 	 */
 	if (vt.mem_flags & MEMORY_X86_PAE) {
-		if (info->kernel_version < VERSION_LINUX_2_6_26)
+		if (info->kernel_version < KERNEL_VERSION(2, 6, 26))
 			info->section_size_bits = _SECTION_SIZE_BITS_PAE_ORIG;
 		else
 			info->section_size_bits = _SECTION_SIZE_BITS_PAE_2_6_26;
@@ -261,8 +261,8 @@ int get_xen_info_x86(void)
 		ERRMSG("Can't get the value of xenheap_phys_end.\n");
 		return FALSE;
 	}
-	info->xen_heap_end = (xen_end >> PAGESHIFT());
 	info->xen_heap_start = 0;
+	info->xen_heap_end   = paddr_to_pfn(xen_end);
 
 	/*
 	 * pickled_id == domain addr for x86
